module SevenSegment (sevenSegment) where

import Graphics.Collage exposing (..)
import Graphics.Element exposing (Element)
import Color exposing (Color)
import String


colorOn : Color
colorOn =
  Color.rgb 255 0 0


colorOff : Color
colorOff =
  Color.rgb 75 0 0


colorBg : Color
colorBg =
  Color.black


drawDigit : Float -> Char -> Form
drawDigit width char =
  let
    bg =
      filled colorOff <| rect width (width * 7 / 4)

    overlay =
      [ outlined (solid colorBg)
          <| rect width (width * 7 / 4)
      , filled colorBg
          <| polygon
              [ ( -width / 4, width * 5 / 8 )
              , ( width / 4, width * 5 / 8 )
              , ( width / 4, width / 8 )
              , ( -width / 4, width / 8 )
              ]
      , filled colorBg
          <| polygon
              [ ( -width / 4, -width * 5 / 8 )
              , ( width / 4, -width * 5 / 8 )
              , ( width / 4, -width / 8 )
              , ( -width / 4, -width / 8 )
              ]
      , traced (solid colorBg)
          <| segment
              ( -width / 2, width * 7 / 8 )
              ( width / 2, -width / 8 )
      , traced (solid colorBg)
          <| segment
              ( width / 2, width * 7 / 8 )
              ( -width / 2, -width / 8 )
      , traced (solid colorBg)
          <| segment
              ( -width / 2, -width * 7 / 8 )
              ( width / 2, width / 8 )
      , traced (solid colorBg)
          <| segment
              ( width / 2, -width * 7 / 8 )
              ( -width / 2, width / 8 )
      , filled colorBg
          <| polygon
              [ ( -width / 2, width / 8 )
              , ( -width * 3 / 8, 0 )
              , ( -width / 2, -width / 8 )
              ]
      , filled colorBg
          <| polygon
              [ ( width / 2, width / 8 )
              , ( width * 3 / 8, 0 )
              , ( width / 2, -width / 8 )
              ]
      , filled colorBg
          <| polygon
              [ ( -width / 2, width * 7 / 8 )
              , ( -width / 4, width * 7 / 8 )
              , ( -width / 2, width * 5 / 8 )
              ]
      , filled colorBg
          <| polygon
              [ ( width / 2, width * 7 / 8 )
              , ( width / 4, width * 7 / 8 )
              , ( width / 2, width * 5 / 8 )
              ]
      , filled colorBg
          <| polygon
              [ ( -width / 2, -width * 7 / 8 )
              , ( -width / 4, -width * 7 / 8 )
              , ( -width / 2, -width * 5 / 8 )
              ]
      , filled colorBg
          <| polygon
              [ ( width / 2, -width * 7 / 8 )
              , ( width / 4, -width * 7 / 8 )
              , ( width / 2, -width * 5 / 8 )
              ]
      ]

    top =
      filled colorOn
        <| polygon
            [ ( -width / 2, width * 7 / 8 )
            , ( width / 2, width * 7 / 8 )
            , ( 0, width * 3 / 8 )
            ]

    upperRight =
      filled colorOn
        <| polygon
            [ ( width / 2, width * 7 / 8 )
            , ( width / 2, -width / 8 )
            , ( 0, width * 3 / 8 )
            ]

    upperLeft =
      filled colorOn
        <| polygon
            [ ( -width / 2, width * 7 / 8 )
            , ( -width / 2, -width / 8 )
            , ( 0, width * 3 / 8 )
            ]

    middle =
      filled colorOn
        <| polygon
            [ ( -width * 3 / 8, 0 )
            , ( 0, width * 3 / 8 )
            , ( width * 3 / 8, 0 )
            , ( 0, -width * 3 / 8 )
            ]

    bottom =
      filled colorOn
        <| polygon
            [ ( -width / 2, -width * 7 / 8 )
            , ( width / 2, -width * 7 / 8 )
            , ( 0, -width * 3 / 8 )
            ]

    lowerRight =
      filled colorOn
        <| polygon
            [ ( width / 2, -width * 7 / 8 )
            , ( width / 2, width / 8 )
            , ( 0, -width * 3 / 8 )
            ]

    lowerLeft =
      filled colorOn
        <| polygon
            [ ( -width / 2, -width * 7 / 8 )
            , ( -width / 2, width / 8 )
            , ( 0, -width * 3 / 8 )
            ]
  in
    (\segs -> group <| bg :: segs ++ overlay)
      <| case char of
          '0' ->
            [ top, upperLeft, lowerRight, bottom, lowerLeft, upperRight ]

          '1' ->
            [ upperRight, lowerRight ]

          '2' ->
            [ top, upperRight, middle, lowerLeft, bottom ]

          '3' ->
            [ top, upperRight, middle, lowerRight, bottom ]

          '4' ->
            [ upperLeft, middle, upperRight, lowerRight ]

          '5' ->
            [ top, upperLeft, middle, lowerRight, bottom ]

          '6' ->
            [ top, upperLeft, middle, lowerRight, bottom, lowerLeft ]

          '7' ->
            [ top, upperRight, lowerRight ]

          '8' ->
            [ top, upperLeft, middle, lowerRight, bottom, lowerLeft, upperRight ]

          '9' ->
            [ top, upperLeft, middle, lowerRight, bottom, upperRight ]

          '-' ->
            [ middle ]

          ' ' ->
            []

          _ ->
            []


sevenSegmentString : Int -> String -> Element
sevenSegmentString height str =
  let
    width =
      toFloat height * 8 / 15

    len =
      String.length str

    resultWidth =
      if len <= 0 then
        0
      else
        width * 10 / 9 * toFloat len - width / 9
  in
    collage
      (ceiling resultWidth)
      height
      [ move ( -resultWidth / 2 + width / 2, 0 )
          <| List.foldr
              (\c f ->
                group
                  [ move ( width * 10 / 9, 0 ) f
                  , filled colorBg
                      <| rect (width * 9 / 8) (width * 15 / 8)
                  , drawDigit width c
                  ]
              )
              (group [])
              (String.toList str)
      ]


sevenSegment : Int -> Int -> Int -> Element
sevenSegment height digits num =
  if digits <= 0 then
    Graphics.Element.empty
  else
    sevenSegmentString height
      <| String.padLeft digits ' '
      <| toString
      <| clamp
          (if digits <= 1 then
            0
           else
            -(10 ^ (digits - 1)) + 1
          )
          (10 ^ digits - 1)
          num
